@@ -15,15 +15,16 @@
  */
 
 
-#include <bson.h>
-#include <bcon.h>
+#include <bson/bson.h>
+#include <bson/bcon.h>
 #define BSON_INSIDE
-#include <bson-private.h>
+#include <bson/bson-private.h>
 #undef BSON_INSIDE
 #include <fcntl.h>
 #include <time.h>
 
 #include "TestSuite.h"
+#include "test-conveniences.h"
 
 /* CDRIVER-2460 ensure the unused old BSON_ASSERT_STATIC macro still compiles */
 BSON_STATIC_ASSERT (1 == 1);
@@ -1248,6 +1249,11 @@ test_bson_validate (void)
                   12,
                   BSON_VALIDATE_NONE,
                   "corrupt BSON");
+   VALIDATE_TEST ("test59.bson",
+                  BSON_VALIDATE_NONE,
+                  9,
+                  BSON_VALIDATE_NONE,
+                  "corrupt BSON");
 
    /* DBRef validation */
    b = BCON_NEW ("my_dbref",
@@ -1401,7 +1407,8 @@ test_bson_utf8_key (void)
    BSON_ASSERT (bson_iter_init (&iter, b));
    BSON_ASSERT (bson_iter_next (&iter));
    BSON_ASSERT (!strcmp (bson_iter_key (&iter), FIVE_EUROS));
-   BSON_ASSERT ((str = bson_iter_utf8 (&iter, &length)));
+   str = bson_iter_utf8 (&iter, &length);
+   BSON_ASSERT (str);
    BSON_ASSERT (length == 15); /* 5 3-byte sequences. */
    BSON_ASSERT (!strcmp (str, FIVE_EUROS));
    bson_destroy (b);
@@ -2171,7 +2178,7 @@ test_bson_visit_unsupported_type (void)
 
    BSON_ASSERT (bson_init_static (&b, (const uint8_t *) data, sizeof data - 1));
    BSON_ASSERT (bson_iter_init (&iter, &b));
-   bson_iter_visit_all (&iter, &visitor, (void *) &context);
+   BSON_ASSERT (!bson_iter_visit_all (&iter, &visitor, (void *) &context));
    BSON_ASSERT (!bson_iter_next (&iter));
    BSON_ASSERT (context.visited);
    BSON_ASSERT (!strcmp (context.key, "k"));
@@ -2193,7 +2200,7 @@ test_bson_visit_unsupported_type_bad_key (void)
 
    BSON_ASSERT (bson_init_static (&b, (const uint8_t *) data, sizeof data - 1));
    BSON_ASSERT (bson_iter_init (&iter, &b));
-   bson_iter_visit_all (&iter, &visitor, (void *) &context);
+   BSON_ASSERT (!bson_iter_visit_all (&iter, &visitor, (void *) &context));
    BSON_ASSERT (!bson_iter_next (&iter));
 
    /* unsupported type error wasn't reported, because the bson is corrupt */
@@ -2215,7 +2222,7 @@ test_bson_visit_unsupported_type_empty_key (void)
 
    BSON_ASSERT (bson_init_static (&b, (const uint8_t *) data, sizeof data - 1));
    BSON_ASSERT (bson_iter_init (&iter, &b));
-   bson_iter_visit_all (&iter, &visitor, (void *) &context);
+   BSON_ASSERT (!bson_iter_visit_all (&iter, &visitor, (void *) &context));
    BSON_ASSERT (!bson_iter_next (&iter));
    BSON_ASSERT (context.visited);
    BSON_ASSERT (!strcmp (context.key, ""));
@@ -2285,6 +2292,89 @@ test_bson_regex_lengths (void)
    bson_destroy (&new);
 }
 
+void
+test_bson_empty_binary (void)
+{
+   uint8_t data = 0xAB;
+   bson_t test;
+   const bson_value_t *value;
+   bson_value_t copy;
+   bson_iter_t iter;
+
+   bson_init (&test);
+   bson_append_binary (&test, "test", 4, BSON_SUBTYPE_BINARY, &data, 0);
+   BSON_ASSERT (bson_iter_init_find (&iter, &test, "test"));
+   value = bson_iter_value (&iter);
+   /* CDRIVER-2569, this would memcpy (0 bytes) to a NULL destination. */
+   bson_value_copy (value, &copy);
+
+   bson_value_destroy (&copy);
+   bson_destroy (&test);
+}
+
+void
+test_bson_iter_key_len (void)
+{
+   bson_t *bson = bson_with_all_types ();
+   bson_iter_t iter;
+
+   BSON_ASSERT (bson_iter_init (&iter, bson));
+   while (bson_iter_next (&iter)) {
+      ASSERT_WITH_MSG (strlen (bson_iter_key (&iter)) ==
+                          bson_iter_key_len (&iter),
+                       "iter_key_len differs from real key length. got %d but "
+                       "expected %d for key %s\n",
+                       bson_iter_key_len (&iter),
+                       (int) strlen (bson_iter_key (&iter)),
+                       bson_iter_key (&iter));
+   }
+}
+
+
+void
+test_bson_iter_init_from_data_at_offset (void)
+{
+   bson_t *bson = bson_with_all_types ();
+   /* zero out iter, since bson_iter_init doesn't zero out iter->value. */
+   bson_iter_t iter = {0};
+
+   BSON_ASSERT (bson_iter_init (&iter, bson));
+   ASSERT_CMPINT (bson_iter_offset (&iter), ==, 0);
+   while (bson_iter_next (&iter)) {
+      const uint8_t *data = bson_get_data (bson);
+      int keylen = bson_iter_key_len (&iter);
+      uint32_t offset = bson_iter_offset (&iter);
+      bson_iter_t recreated = {0};
+
+      BSON_ASSERT (bson_iter_init_from_data_at_offset (
+         &recreated, data, bson->len, offset, keylen));
+      if (memcmp ((void *) &iter, (void *) &recreated, sizeof (bson_iter_t)) !=
+          0) {
+         int i;
+         bson_iter_t *iters[] = {&iter, &recreated};
+         fprintf (stderr,
+                  "recreated iterator does not match initial iterator:\n");
+         for (i = 0; i < 2; i++) {
+            fprintf (stderr, "iter %d: ", i);
+            fprintf (stderr,
+                     "len=%d, off=%d, type=%d, key=%d, d1=%d, d2=%d, "
+                     "d3=%d, d4=%d, next_off=%d, err_off=%d\n",
+                     iters[i]->len,
+                     iters[i]->off,
+                     iters[i]->type,
+                     iters[i]->key,
+                     iters[i]->d1,
+                     iters[i]->d2,
+                     iters[i]->d3,
+                     iters[i]->d4,
+                     iters[i]->next_off,
+                     iters[i]->err_off);
+         }
+         ASSERT (false);
+      }
+   }
+}
+
 void
 test_bson_install (TestSuite *suite)
 {
@@ -2372,4 +2462,9 @@ test_bson_install (TestSuite *suite)
    TestSuite_Add (suite, "/bson/binary_subtype_2", test_bson_subtype_2);
    TestSuite_Add (suite, "/bson/regex_length", test_bson_regex_lengths);
    TestSuite_Add (suite, "/util/next_power_of_two", test_next_power_of_two);
+   TestSuite_Add (suite, "/bson/empty_binary", test_bson_empty_binary);
+   TestSuite_Add (suite, "/bson/iter/key_len", test_bson_iter_key_len);
+   TestSuite_Add (suite,
+                  "/bson/iter/init_from_data_at_offset",
+                  test_bson_iter_init_from_data_at_offset);
 }
