@@ -15,14 +15,14 @@
  */
 
 
-#include "mongoc-config.h"
-#include "mongoc-collection-private.h"
-#include "mongoc-host-list-private.h"
-#include "mongoc-server-description-private.h"
-#include "mongoc-topology-description-private.h"
-#include "mongoc-topology-private.h"
-#include "mongoc-util-private.h"
-#include "mongoc-util-private.h"
+#include "mongoc/mongoc-config.h"
+#include "mongoc/mongoc-collection-private.h"
+#include "mongoc/mongoc-host-list-private.h"
+#include "mongoc/mongoc-server-description-private.h"
+#include "mongoc/mongoc-topology-description-private.h"
+#include "mongoc/mongoc-topology-private.h"
+#include "mongoc/mongoc-util-private.h"
+#include "mongoc/mongoc-util-private.h"
 
 #include "TestSuite.h"
 #include "test-conveniences.h"
@@ -98,11 +98,6 @@ convert_message_for_test (json_test_ctx_t *ctx,
    char *child_path;
    bson_t lsid;
 
-   if (bson_empty (src) && !ctx->acknowledged) {
-      /* spec tests say unacknowledged writes reply "ok": 1, but we don't */
-      BSON_APPEND_DOUBLE (dst, "ok", 1.0);
-      return;
-   }
 
    if (!path && !bson_empty (src)) {
       const char *cmd_name = _mongoc_get_command_name (src);
@@ -112,7 +107,7 @@ convert_message_for_test (json_test_ctx_t *ctx,
       }
    }
 
-   bson_iter_init (&iter, src);
+   BSON_ASSERT (bson_iter_init (&iter, src));
 
    while (bson_iter_next (&iter)) {
       key = bson_iter_key (&iter);
@@ -433,10 +428,15 @@ set_apm_callbacks (mongoc_client_t *client,
  *   }
  * ]
  *
+ *      If @allow_subset is true, then expectations is allowed to be
+ *      a subset of events.
+ *
  *-----------------------------------------------------------------------
  */
 void
-check_json_apm_events (const bson_t *events, const bson_t *expectations)
+check_json_apm_events (const bson_t *events,
+                       const bson_t *expectations,
+                       bool allow_subset)
 {
    char errmsg[1000] = {0};
    match_ctx_t ctx = {0};
@@ -446,33 +446,44 @@ check_json_apm_events (const bson_t *events, const bson_t *expectations)
    /* Old mongod returns a double for "count", newer returns int32.
     * Ignore this and other insignificant type differences. */
    ctx.strict_numeric_types = false;
+   ctx.retain_dots_in_keys = true;
    ctx.errmsg = errmsg;
    ctx.errmsg_len = sizeof errmsg;
 
-   expected_keys = bson_count_keys (expectations);
-   actual_keys = bson_count_keys (events);
+   if (!allow_subset) {
+      expected_keys = bson_count_keys (expectations);
+      actual_keys = bson_count_keys (events);
 
-   if (expected_keys != actual_keys) {
-      test_error ("command monitoring test failed expectations:\n\n"
-                  "%s\n\n"
-                  "events:\n%s\n\n"
-                  "expected %" PRIu32 " events, got %" PRIu32,
-                  bson_as_canonical_extended_json (expectations, NULL),
-                  bson_as_canonical_extended_json (events, NULL),
-                  expected_keys,
-                  actual_keys);
+      if (expected_keys != actual_keys) {
+         test_error ("command monitoring test failed expectations:\n\n"
+                     "%s\n\n"
+                     "events:\n%s\n\n"
+                     "expected %" PRIu32 " events, got %" PRIu32,
+                     bson_as_canonical_extended_json (expectations, NULL),
+                     bson_as_canonical_extended_json (events, NULL),
+                     expected_keys,
+                     actual_keys);
 
-      abort ();
-   }
+         abort ();
+      }
 
-   if (!match_bson_with_ctx (events, expectations, false, &ctx)) {
-      test_error ("command monitoring test failed expectations:\n\n"
-                  "%s\n\n"
-                  "events:\n%s\n\n%s",
-                  bson_as_canonical_extended_json (expectations, NULL),
-                  bson_as_canonical_extended_json (events, NULL),
-                  errmsg);
+      if (!match_bson_with_ctx (events, expectations, false, &ctx)) {
+         test_error ("command monitoring test failed expectations:\n\n"
+                     "%s\n\n"
+                     "events:\n%s\n\n%s",
+                     bson_as_canonical_extended_json (expectations, NULL),
+                     bson_as_canonical_extended_json (events, NULL),
+                     errmsg);
+      }
+   } else {
+      bson_iter_t expectations_iter;
+      BSON_ASSERT (bson_iter_init (&expectations_iter, expectations));
 
-      abort ();
+      while (bson_iter_next (&expectations_iter)) {
+         bson_t expectation;
+         bson_iter_bson (&expectations_iter, &expectation);
+         match_in_array (&expectation, events, &ctx);
+         bson_destroy (&expectation);
+      }
    }
 }
