@@ -19,8 +19,8 @@
 
 #include "TestSuite.h"
 
-#include <bson.h>
-#include <mongoc.h>
+#include <bson/bson.h>
+#include <mongoc/mongoc.h>
 
 #include "json-test-operations.h"
 
@@ -32,6 +32,8 @@ set_apm_callbacks (mongoc_client_t *client,
                    void *ctx);
 
 void
-check_json_apm_events (const bson_t *events, const bson_t *expectations);
+check_json_apm_events (const bson_t *events,
+                       const bson_t *expectations,
+                       bool allow_subset);
 
 #endif
