@@ -15,17 +15,17 @@
  */
 
 
-#include "bson.h"
+#include "bson/bson.h"
 
-#include "mongoc-collection-private.h"
-#include "mongoc-config.h"
-#include "mongoc-cursor-private.h"
-#include "mongoc-host-list-private.h"
-#include "mongoc-server-description-private.h"
-#include "mongoc-topology-description-private.h"
-#include "mongoc-topology-private.h"
-#include "mongoc-util-private.h"
-#include "mongoc-util-private.h"
+#include "mongoc/mongoc-collection-private.h"
+#include "mongoc/mongoc-config.h"
+#include "mongoc/mongoc-cursor-private.h"
+#include "mongoc/mongoc-host-list-private.h"
+#include "mongoc/mongoc-server-description-private.h"
+#include "mongoc/mongoc-topology-description-private.h"
+#include "mongoc/mongoc-topology-private.h"
+#include "mongoc/mongoc-util-private.h"
+#include "mongoc/mongoc-util-private.h"
 
 #include "json-test-operations.h"
 #include "json-test.h"
@@ -401,7 +401,7 @@ check_error_labels_contain (const bson_t *operation, const bson_value_t *result)
       return;
    }
 
-   bson_iter_init (&operation_iter, operation);
+   BSON_ASSERT (bson_iter_init (&operation_iter, operation));
    BSON_ASSERT (bson_iter_find_descendant (
       &operation_iter, "result.errorLabelsContain", &expected_labels));
    BSON_ASSERT (bson_iter_recurse (&expected_labels, &expected_label));
@@ -576,6 +576,13 @@ add_request_to_bulk (mongoc_bulk_operation_t *bulk,
    name = bson_lookup_utf8 (request, "name");
    bson_lookup_doc (request, "arguments", &args);
 
+   if (bson_has_field (&args, "arrayFilters")) {
+      bson_value_t array_filters;
+      bson_lookup_value (&args, "arrayFilters", &array_filters);
+      BSON_APPEND_VALUE (&opts, "arrayFilters", &array_filters);
+      bson_value_destroy (&array_filters);
+   }
+
    if (!strcmp (name, "deleteMany")) {
       bson_t filter;
 
@@ -845,6 +852,26 @@ create_find_and_modify_opts (const char *name,
       flags |= MONGOC_FIND_AND_MODIFY_UPSERT;
    }
 
+   if (bson_has_field (args, "collation")) {
+      bson_t collation = BSON_INITIALIZER;
+      bson_t temp;
+      bson_lookup_doc (args, "collation", &temp);
+      BSON_APPEND_DOCUMENT (&collation, "collation", &temp);
+      mongoc_find_and_modify_opts_append (opts, &collation);
+      bson_destroy (&collation);
+      bson_destroy (&temp);
+   }
+
+   if (bson_has_field (args, "arrayFilters")) {
+      bson_t array_filters = BSON_INITIALIZER;
+      bson_value_t temp;
+      bson_lookup_value (args, "arrayFilters", &temp);
+      BSON_APPEND_VALUE (&array_filters, "arrayFilters", &temp);
+      mongoc_find_and_modify_opts_append (opts, &array_filters);
+      bson_destroy (&array_filters);
+      bson_value_destroy (&temp);
+   }
+
    if (bson_has_field (args, "returnDocument") &&
        !strcmp ("After", bson_lookup_utf8 (args, "returnDocument"))) {
       flags |= MONGOC_FIND_AND_MODIFY_RETURN_NEW;
@@ -1322,17 +1349,16 @@ void
 json_test_operation (json_test_ctx_t *ctx,
                      const bson_t *test,
                      const bson_t *operation,
+                     mongoc_collection_t *collection,
                      mongoc_client_session_t *session)
 {
    const char *op_name;
    mongoc_read_prefs_t *read_prefs = NULL;
    mongoc_write_concern_t *wc;
-   mongoc_collection_t *collection;
 
    op_name = bson_lookup_utf8 (operation, "name");
    /* databaseOptions don't yet exist in tests, therefore not implemented */
    BSON_ASSERT (!bson_has_field (operation, "databaseOptions"));
-   collection = mongoc_collection_copy (ctx->collection);
    if (bson_has_field (operation, "collectionOptions")) {
       bson_lookup_collection_opts (operation, "collectionOptions", collection);
    }
@@ -1391,7 +1417,6 @@ json_test_operation (json_test_ctx_t *ctx,
       test_error ("unrecognized operation name %s", op_name);
    }
 
-   mongoc_collection_destroy (collection);
    mongoc_read_prefs_destroy (read_prefs);
    mongoc_write_concern_destroy (wc);
 }
@@ -1449,7 +1474,6 @@ json_test_operations (json_test_ctx_t *ctx, const bson_t *test)
       one_operation (ctx, test, &operation);
    } else {
       bson_lookup_doc (test, "operations", &operations);
-      ASSERT_CMPUINT32 (bson_count_keys (&operations), >, (uint32_t) 0);
       BSON_ASSERT (bson_iter_init (&iter, &operations));
       while (bson_iter_next (&iter)) {
          bson_iter_bson (&iter, &operation);
