@@ -15,8 +15,8 @@
  */
 
 
-#include <mongoc-rpc-private.h>
-#include "mongoc.h"
+#include <mongoc/mongoc-rpc-private.h>
+#include "mongoc/mongoc.h"
 
 #include "mock-server.h"
 #include "../test-conveniences.h"
@@ -569,7 +569,7 @@ request_matches_msg (const request_t *request,
       doc = request_get_doc (request, i);
       /* pass is_command=true for first doc, including "find" command */
       is_command_doc = (i == 0);
-      match_bson (doc, pattern, is_command_doc);
+      BSON_ASSERT (match_bson (doc, pattern, is_command_doc));
    }
 
    if (n_docs < request->docs.len) {
@@ -1097,6 +1097,7 @@ request_from_op_msg (request_t *request, const mongoc_rpc_t *rpc)
    bson_iter_t iter;
    bson_string_t *msg_as_str = bson_string_new ("OP_MSG");
 
+   BSON_ASSERT (rpc->msg.n_sections <= 2);
    for (section_no = 0; section_no < rpc->msg.n_sections; section_no++) {
       bson_string_append (msg_as_str, (section_no > 0 ? ", " : " "));
       section = &rpc->msg.sections[section_no];
@@ -1132,8 +1133,7 @@ request_from_op_msg (request_t *request, const mongoc_rpc_t *rpc)
 
    if (request->docs.len) {
       doc = request_get_doc (request, 0);
-      bson_iter_init (&iter, doc);
-      if (bson_iter_next (&iter)) {
+      if (bson_iter_init (&iter, doc) && bson_iter_next (&iter)) {
          request->command_name = bson_strdup (bson_iter_key (&iter));
       }
    }
