@@ -1,10 +1,10 @@
 #include <fcntl.h>
-#include <mongoc.h>
+#include <mongoc/mongoc.h>
 
-#include "mongoc-client-private.h"
-#include "mongoc-cursor-private.h"
-#include "mongoc-uri-private.h"
-#include "mongoc-util-private.h"
+#include "mongoc/mongoc-client-private.h"
+#include "mongoc/mongoc-cursor-private.h"
+#include "mongoc/mongoc-uri-private.h"
+#include "mongoc/mongoc-util-private.h"
 
 #include "TestSuite.h"
 #include "test-conveniences.h"
@@ -85,7 +85,8 @@ test_exhaust_cursor (bool pooled)
    collection = get_test_collection (client, "test_exhaust_cursor");
    BSON_ASSERT (collection);
 
-   mongoc_collection_drop (collection, &error);
+   /* don't care if ns not found. */
+   (void) mongoc_collection_drop (collection, &error);
 
    wr = mongoc_write_concern_new ();
    mongoc_write_concern_set_journal (wr, true);
@@ -208,7 +209,7 @@ test_exhaust_cursor (bool pooled)
     * 1. check that we can create a new cursor, as long as we don't read from it
     * 2. fully exhaust the exhaust cursor
     * 3. make sure that we don't disconnect at destroy
-    * 4. make sure we can read the cursor we made during the exhuast
+    * 4. make sure we can read the cursor we made during the exhaust
     */
    {
       cursor2 = mongoc_collection_find (
