@@ -1,8 +1,8 @@
-#include <bcon.h>
-#include <mongoc.h>
-#include <mongoc-collection-private.h>
-#include <mongoc-find-and-modify-private.h>
-#include <mongoc-client-private.h>
+#include <bson/bcon.h>
+#include <mongoc/mongoc.h>
+#include <mongoc/mongoc-collection-private.h>
+#include <mongoc/mongoc-find-and-modify-private.h>
+#include <mongoc/mongoc-client-private.h>
 
 #include "TestSuite.h"
 
@@ -112,8 +112,7 @@ test_find_and_modify_bypass (bool bypass)
          "{ 'findAndModify' : 'test_find_and_modify', "
          "'query' : { 'superduper' : 77889 },"
          "'update' : { '$set' : { 'superduper' : 1234 } },"
-         "'new' : true,"
-         "'bypassDocumentValidation' : false }");
+         "'new' : true }");
    }
 
    mock_server_replies_simple (request, "{ 'value' : null, 'ok' : 1 }");
@@ -511,7 +510,7 @@ test_find_and_modify_collation (int wire)
                          BCON_UTF8 ("lower"),
                          "}");
    opts = mongoc_find_and_modify_opts_new ();
-   mongoc_find_and_modify_opts_append (opts, collation);
+   BSON_ASSERT (mongoc_find_and_modify_opts_append (opts, collation));
 
    if (wire >= WIRE_VERSION_COLLATION) {
       future = future_collection_find_and_modify_with_opts (
